#ifndef MOON_RISE_H
#define MOON_RISE_H

#include "CalcEphem.h"

void MoonRise(CTrans * c, gdouble * UTRise, gdouble * UTSet);
void UTTohhmm(gdouble UT, gint * h, gint * m);

#endif				/* MOON_RISE_H */
